Index: iputils-20210202/arping.c
===================================================================
--- iputils-20210202.orig/arping.c
+++ iputils-20210202/arping.c
@@ -66,6 +66,8 @@ struct run_state {
 	int timeout;
 	unsigned int interval;
 	int socketfd;
+	char *padding;
+	unsigned int padlen;
 	struct sockaddr_storage me;
 	struct sockaddr_storage he;
 	struct timespec start;
@@ -126,6 +128,39 @@ __attribute__((const)) static inline siz
 	return len;
 }
 
+u_int32_t format_hex_payload(char *string)
+{
+	char *pl;
+	char *i, *delim = " ";
+	char tchar[2];
+	long c;
+	u_int32_t len = 0;
+
+#ifdef DEBUG
+	fprintf(stdout, "DEBUG: format_hex_payload()\n");
+#endif
+
+	pl = strdup(string);
+	pl[0] = pl[1] = 20;
+
+	memset(string, 0, strlen(string));
+	memset(tchar, 0, 2);
+
+	/*
+	 * skip the first 3 chars because we know they are spaces
+	 */
+	for(i = strtok(pl+3, delim); i; i = strtok(NULL, delim)) {
+		if((c = strtol(i, 0, 16)) > 0xff)
+			return 0;
+
+		sprintf(tchar,"%c",(u_int8_t)c);
+		strncpy(string+len,tchar,2);
+		len++;
+	}
+
+	return len;
+}
+
 static void usage(void)
 {
 	fprintf(stderr, _(
@@ -145,6 +180,7 @@ static void usage(void)
 		"  -u            with -T, dont send tagged packets\n"
 		"  -c <count>    how many packets to send\n"
 		"  -C <count>    how many packets to receive (1 == -f)\n"
+		"  -p <padstr>   packet padding ascii string, or \"0x 11 22 33 44\" hex\n"
 		"  -w <timeout>  how long to wait for a reply\n"
 		"  -i <interval> set interval between packets (default: 1 second)\n"
 		"  -I <device>   which ethernet device to use"
@@ -450,6 +486,10 @@ static int send_pack(struct run_state *c
 	memcpy(p, &ctl->gdst, 4);
 	p += 4;
 
+	if (ctl->padlen)
+		memcpy(p, ctl->padding, ctl->padlen);
+	p += ctl->padlen;
+
 	clock_gettime(CLOCK_MONOTONIC, &now);
 	err = sendto(ctl->socketfd, buf, p - buf, 0, (struct sockaddr *)HE, sll_len(ah->ar_hln));
 	if (err == p - buf) {
@@ -1125,7 +1165,7 @@ int main(int argc, char **argv)
 	textdomain (PACKAGE_NAME);
 #endif
 #endif
-	while ((ch = getopt(argc, argv, "h?bfDuUAqc:w:i:Ss:I:VtT:")) != EOF) {
+	while ((ch = getopt(argc, argv, "h?bfDuUAqc:w:i:Ss:I:VtT:p:")) != EOF) {
 		switch (ch) {
 		case 'b':
 			ctl.broadcast_only = 1;
@@ -1153,6 +1193,24 @@ int main(int argc, char **argv)
 		case 'w':
 			ctl.timeout = strtol_or_err(optarg, _("invalid argument"), 0, INT_MAX);
 			break;
+		case 'p':
+			if(!(ctl.padding = strdup(optarg)))
+				error(2, errno, "padding strdup");
+			ctl.padlen = strlen(ctl.padding);
+			if(!strncasecmp(optarg, "0x", 2))
+			{
+				if (ctl.padlen < 4)
+				{
+					error(2, errno,  "Padding hex string is blank");
+					usage();
+					exit(1);
+				}
+				if((ctl.padlen = format_hex_payload(ctl.padding)) < 1)
+					error(2, errno, "hex padding");
+			}
+			if (ctl.padlen > 384)
+				error(2, 0, "Padding over maximum 384 length");
+			break;
 		case 'i':
 			ctl.interval = strtol_or_err(optarg, _("invalid argument"), 0, INT_MAX);
 			break;
